#include <algorithm>
#include <cassert>
#include <iostream>
#include <random>
#include <vector>
using namespace std;

/* Generates cases where multiple people tie for first if the ranks in the
 * third round are assigned as in the model solution (i.e. by decreasing
 * product of the first two ranks). This amounts to finding values a[i]
 * and b[i] such that:
 *
 * - a[2]*b[2] >= ... >= a[N]*b[N];
 * - a[1]*b[1] <= a[k]*b[k]*k for each 2 <= k <= n;
 * - a[1]*b[1] == a[k]*b[k]*k as often as possible.
 *
 * The general approach here is to choose the values k in the third item to all
 * be near the boundaries, i.e. either close to 1 or close to n. In the code
 * below, L is the largest among the small values for k, K is the smallest
 * among the large values, and X = a[1]*b[1]. For i <= L and i >= K possible
 * values of a[i] and b[i] are brute forced, and bipartite matching is used for
 * the rest.
 */

struct matching {
	int L, R;
	vector<vector<int>> adj;
	vector<int> matchR, mark;

	matching(int L, int R): L(L), R(R), adj(L), matchR(R,-1) { }

	bool dfs(int i) {
		if (i == -1) return true;
		if (mark[i]) return false;
		mark[i] = true;
		for (int j: adj[i]) if (dfs(matchR[j])) {
			matchR[j] = i;
			return true;
		}
		return false;
	}
	
	int max_match() {
		int res = 0;
		for (int i = 0; i < L; i++) {
			mark.assign(L, false);
			if (dfs(i)) res++;
		}
		return res;
	}
};

mt19937_64 gen(987);

int N, L, K, X;

vector<int> divisors[10005];
int a[101], b[101];
bool useda[101], usedb[101];

void match_rest() {
	int lo = a[K]*b[K], hi = a[L]*b[L];
	matching M(N+1, N+1);
	for (int u = 1; u <= N; u++) if (!useda[u]) {
		for (int v = 1; v <= N; v++) if (!usedb[v]) {
			if (lo <= u*v && u*v <= hi) M.adj[u].push_back(v);
		}
	}

	if (M.max_match() < K-L-1) return;
	int i = L+1;
	for (int v = 1; v <= N; v++) if (M.matchR[v] != -1) {
		a[i] = M.matchR[v], b[i] = v, i++;
	}

	cerr << N << " " << L << " " << K << " " << X << endl;

	pair<int,int> ab[N+1];
	for (int k = 1; k <= N; k++) ab[k] = {a[k], b[k]};
	shuffle(ab+2, ab+N+1, gen);

	cout << N << endl;
	for (int k = 1; k <= N; k++) cout << ab[k].first  << " \n"[k==N];
	for (int k = 1; k <= N; k++) cout << ab[k].second << " \n"[k==N];
	exit(0);
}

void rec(int k, int y, int m) {
	if (m == 0 && k > N) return rec(1, X, 1);
	if (m == 1 && k > L) return match_rest();
	if (X%k == 0) y = X/k;
	for (; k*y >= X; y--) {
		for (int u: divisors[y]) {
			if (useda[u] || usedb[y/u]) continue;
			a[k] = u, b[k] = y/u;
			useda[u] = usedb[y/u] = 1;
			rec(k+1, y, m);
			useda[u] = usedb[y/u] = 0;
			a[k] = b[k] = 0;
		}
	}
}

void find_candidates(int s, int t) {
	for (X = N*N; X >= 1; X--) {
		vector<int> all_divisors;
		for (int x = 1; x <= N; x++) {
			if (X%x == 0) all_divisors.push_back(x);
		}
		if (ssize(all_divisors) < s) continue;
		L = all_divisors[s-1];

		if (ssize(divisors[X]) < t) continue;
		K = divisors[X][size(divisors[X])-t];
		
		cerr << N << " " << L << " " << K << " " << X << endl;
		if (L < K) rec(K, N*N, 0);
	}
}

int main(int argc, char **argv) {
	assert(argc >= 5);
	N = atoi(argv[1]);
	L = atoi(argv[2]);
	K = atoi(argv[3]);
	X = atoi(argv[4]);
	
	for (int x = 1; x <= N; x++) {
		for (int y = 1; y <= N; y++) {
			divisors[x*y].push_back(x);
		}
	}
	
	rec(K, N*N, 0);
}
